@@ -24,7 +24,7 @@
   @author Samuel Mimram, and many others...
   *)
 
-(* $Id: wav2ogg.ml,v 1.5 2005/05/23 21:35:44 smimram Exp $ *)
+(* $Id: wav2ogg.ml 2097 2005-05-23 21:35:44Z smimram $ *)
 
 open Unix
 
@@ -38,6 +38,7 @@ let genre = ref None
 let date = ref None
 let album = ref None
 let tracknum = ref None
+let comment = ref None
 let buflen = ref 1024
 
 let input_string chan len =
@@ -88,14 +89,13 @@ let _ =
     );
   let ic = open_in_bin !src in
   let oc = open_out_bin !dst in
-  let r = ref 1 in
     (* TODO: improve! *)
     if input_string ic 4 <> "RIFF" then invalid_arg "No RIFF tag";
     ignore (input_string ic 4);
     if input_string ic 4 <> "WAVE" then invalid_arg "No WAVE tag";
     if input_string ic 4 <> "fmt " then invalid_arg "No fmt tag";
     let _ = input_int ic in
-    let wav_type = input_short ic in (* TODO: should be 1 *)
+    let _ = input_short ic in (* TODO: should be 1 *)
     let channels = input_short ic in
     let infreq = input_int ic in
     let _ = input_int ic in (* bytes / s *)
@@ -117,7 +117,7 @@ let _ =
       } in
     let enc, header =
       Vorbis.create_encoder_opt
-        !title !artist !genre !date !album !tracknum enc_params
+        !title !artist !genre !date !album !tracknum !comment enc_params
     in
     let start = Unix.time () in
       Printf.printf
