@@ -1,5 +1,5 @@
 (*
- Copyright 2003-2004 Savonet team
+ Copyright 2003-2006 Savonet team
 
  This file is part of Ocaml-vorbis.
 
@@ -24,7 +24,7 @@
   @author Samuel Mimram, Julien Cristau, David Baelde
   *)
 
-(* $Id: vorbis.ml,v 1.47 2005/05/26 17:32:10 smimram Exp $ *)
+(* $Id: vorbis.ml 2584 2006-07-06 10:02:58Z dbaelde $ *)
 
 (* TODO: there are too many assert(). Some of them should be handled
  more gracefully *)
@@ -81,13 +81,13 @@ type enc_params =
       enc_in_big_endian : bool;
     }
 
-external create_encoder_pre : enc_params -> string option -> string option -> string option -> string option -> string option -> string option -> encoder*string = "ocaml_vorbis_create_encoder_byte" "ocaml_vorbis_create_encoder"
+external create_encoder_pre : enc_params -> string option -> string option -> string option -> string option -> string option -> string option -> string option -> encoder*string = "ocaml_vorbis_create_encoder_byte" "ocaml_vorbis_create_encoder"
 
-let create_encoder ?title ?artist ?genre ?date ?album ?tracknum params =
-  create_encoder_pre params title artist genre date album tracknum
+let create_encoder ?title ?artist ?genre ?date ?album ?tracknum ?comment params =
+  create_encoder_pre params title artist genre date album tracknum comment
 
-let create_encoder_opt title artist genre date album tracknum params =
-  create_encoder_pre params title artist genre date album tracknum
+let create_encoder_opt title artist genre date album tracknum comment params =
+  create_encoder_pre params title artist genre date album tracknum comment
 
 external encode_buffer_part : encoder -> string -> int -> int -> string = "ocaml_vorbis_encode_buffer"
 
@@ -116,9 +116,9 @@ external decode : dec_file -> string -> int -> int -> int = "ocaml_vorbis_decode
 external close_dec_file : dec_file -> unit = "ocaml_vorbis_close_dec_file"
 
 
-external encoder_reset_pre : encoder -> string option -> string option -> string option -> string option -> string option -> string option -> string = "ocaml_vorbis_encoder_reset_byte" "ocaml_vorbis_encoder_reset"
-let encoder_reset ?title ?artist ?genre ?date ?album ?tracknum ep = encoder_reset_pre ep title artist genre date album tracknum
-let encoder_reset_opt title artist genre date album tracknum ep = encoder_reset_pre ep title artist genre date album tracknum
+external encoder_reset_pre : encoder -> string option -> string option -> string option -> string option -> string option -> string option -> string option -> string = "ocaml_vorbis_encoder_reset_byte" "ocaml_vorbis_encoder_reset"
+let encoder_reset ?title ?artist ?genre ?date ?album ?tracknum ?comment ep = encoder_reset_pre ep title artist genre date album tracknum comment
+let encoder_reset_opt title artist genre date album tracknum comment ep = encoder_reset_pre ep title artist genre date album tracknum comment
 
 
 type info = {
@@ -135,8 +135,15 @@ type info = {
 
 external get_dec_file_bitstream : dec_file -> int = "ocaml_vorbis_get_dec_file_bitstream"
 
+(** Set charset to encode from / decode to. *)
+external utf8_set_charset : string -> unit = "ocaml_vorbis_utf8_set_charset"
+
+let set_charset = utf8_set_charset
+
 external utf8_decode : string -> string = "ocaml_vorbis_utf8_decode"
 
+external utf8_encode : string -> string = "ocaml_vorbis_utf8_encode"
+
 external get_dec_file_comments_ : dec_file -> int option -> string * (string array) = "ocaml_vorbis_get_dec_file_comments"
 
 (* TODO: share this with get_comments *)
@@ -322,7 +329,7 @@ let file = Unix.openfile filename [Unix.O_RDONLY; Unix.O_NONBLOCK] 0o400 in
                                              done;
                                              vendor,comments
 
-let get_info filename =
+let get_info ?(duration=true) filename =
   let file = Unix.openfile filename [Unix.O_RDONLY;Unix.O_NONBLOCK] 0o400 in
   let x = Unix.lseek file 35 Unix.SEEK_SET in
   let () = assert(x = 35) in
@@ -350,8 +357,12 @@ let get_info filename =
         bitrate_minimum = if br_min>0 then Some br_min else None;
         blocksize_0 = blocksize_0;
         blocksize_1 = blocksize_1;
-        duration = Int64.to_int(Int64.div (samples filename)
-                                  (Int64.of_int sample_rate));
+        duration =
+          if duration then
+            Int64.to_int
+              (Int64.div (samples filename) (Int64.of_int sample_rate))
+          else
+            -1
       }
 
 let file_size filename =
@@ -386,7 +397,7 @@ struct
 
   (* Find the lacing values of the segments in the current ogg page *)
   let get_lacing_values file =
-    let x = Io.lseek file 26 Io.SEEK_CUR in
+    let _ = Io.lseek file 26 Io.SEEK_CUR in
     let buf = String.create 1 in
       if Io.read file buf 0 1 <> 1 then raise End_of_file;
       let nb_segments = int_of_char buf.[0] in
@@ -576,7 +587,7 @@ struct
     let r = Str.regexp "OggS" in
     let file = Io.openfile filename [Io.O_RDONLY] 0o400 in
     let filesize = Io.lseek file 0 Io.SEEK_END in
-    let offset =
+    let _ =
       if filesize < size then Io.lseek file 0 Io.SEEK_SET
       else Io.lseek file (-size) Io.SEEK_END
     in
