@@ -1,5 +1,5 @@
 /*
-   Copyright 2003-2004 The Savonet Team
+   Copyright 2003-2006 The Savonet Team
    Some parts from oggenc/oggdec, (c) 2000-2002 Michael Smith
    Some parts from ices2 (c) 2001 Michael Smith <msmith@labyrinth.net.au>
    See COPYRIGHT file for details.
@@ -27,7 +27,7 @@
  * @author Samuel Mimram, Julien Cristau, David Baelde
  */
 
-/* $Id: vorbis_stubs.c,v 1.50 2005/10/16 20:09:11 smimram Exp $ */
+/* $Id: vorbis_stubs.c 2564 2006-07-04 12:57:08Z smimram $ */
 
 /*
 TODO:
@@ -175,7 +175,7 @@ static void add_tag(vorbis_comment *vc, char *name, char *val)
 }
 
 static void build_comments(vorbis_comment *vc,
-    char *artist, char *album, char *title, char *tracknum,
+    char *artist, char *album, char *title, char *tracknum, char* comment,
     char *date, char *genre)
 {
   vorbis_comment_init(vc);
@@ -192,14 +192,16 @@ static void build_comments(vorbis_comment *vc,
     add_tag(vc, "album", album);
   if (tracknum)
     add_tag(vc, "tracknumber", tracknum);
+  if (comment)
+    add_tag(vc, "comment", comment);
   add_tag(vc, "encoder", "ocaml-vorbis by the savonet team (http://savonet.sf.net/)");
 }
 
 
-static value ocaml_vorbis_get_header_frame(value encoder, value title, value artist, value genre, value date, value album, value tracknum)
+static __inline__ value ocaml_vorbis_get_header_frame(value encoder, value title, value artist, value genre, value date, value album, value tracknum, value comment)
 {
   /* CAMLparam5(encoder, title, artist, genre, date);
-  CAMLxparam2(album, tracknum); */
+  CAMLxparam3(album, tracknum, comment); */
   CAMLlocal1(s);
 
   char *ret = NULL;
@@ -214,7 +216,7 @@ static value ocaml_vorbis_get_header_frame(value encoder, value title, value art
     ogg_packet header_codebooks;
     vorbis_comment comments;
 
-    build_comments(&comments, string_of_option(artist), string_of_option(album), string_of_option(title), string_of_option(tracknum), string_of_option(date), string_of_option(genre));
+    build_comments(&comments, string_of_option(artist), string_of_option(album), string_of_option(title), string_of_option(tracknum), string_of_option(comment), string_of_option(date), string_of_option(genre));
     comments.vendor = malloc(strlen("ocaml-vorbis") + 1);
     strcpy(comments.vendor, "ocaml-vorbis");
 
@@ -244,10 +246,10 @@ static value ocaml_vorbis_get_header_frame(value encoder, value title, value art
   return s;
 }
 
-CAMLprim value ocaml_vorbis_create_encoder(value params, value title, value artist, value genre, value date, value album, value tracknum)
+CAMLprim value ocaml_vorbis_create_encoder(value params, value title, value artist, value genre, value date, value album, value tracknum, value comment)
 {
   CAMLparam5(params, title, artist, genre, date);
-  CAMLxparam2(album, tracknum);
+  CAMLxparam3(album, tracknum, comment);
   CAMLlocal2(encoder, block);
 
   myvorbis_encoder_t *ve;
@@ -352,14 +354,14 @@ CAMLprim value ocaml_vorbis_create_encoder(value params, value title, value arti
   block = caml_alloc_tuple(2);
   Store_field(block, 0, encoder);
   /* TODO : clean this */
-  Store_field(block, 1, ocaml_vorbis_get_header_frame(encoder, title, artist, genre, date, album, tracknum));
+  Store_field(block, 1, ocaml_vorbis_get_header_frame(encoder, title, artist, genre, date, album, tracknum, comment));
 
   CAMLreturn(block);
 }
 
 CAMLprim value ocaml_vorbis_create_encoder_byte(value* argv, int argc)
 {
-  return (ocaml_vorbis_create_encoder(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]));
+  return (ocaml_vorbis_create_encoder(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7]));
 }
 
 /* Encode audio from buf, return the available ogg/vorbis data */
@@ -530,7 +532,11 @@ CAMLprim value ocaml_vorbis_open_dec_file(value fd, value params)
   df->seek_func = (value)NULL;
   df->close_func = (value)NULL;
   df->tell_func = (value)NULL;
+
+  caml_enter_blocking_section();
   ret = ov_open(in, df->ovf, NULL, 0);
+  caml_leave_blocking_section();
+
   if (ret < 0)
   {
     fclose(in);
@@ -602,6 +608,7 @@ CAMLprim value ocaml_vorbis_open_dec_stream(value read_func, value seek_func, va
   df->tell_func = tell_func;
   caml_register_global_root(&df->tell_func);
   ret = ov_open_callbacks(df, df->ovf, NULL, 0, callbacks);
+
   if(ret < 0)
   {
     free(df->ovf);
@@ -660,6 +667,15 @@ CAMLprim value ocaml_vorbis_close_dec_file(value d_f)
   CAMLreturn(Val_unit);
 }
 
+CAMLprim value ocaml_vorbis_utf8_set_charset(value charset)
+{
+  CAMLparam1(charset);
+
+  convert_set_charset(String_val(charset));
+
+  CAMLreturn(Val_unit);
+}
+
 CAMLprim value ocaml_vorbis_utf8_decode(value string)
 {
   CAMLparam1(string);
@@ -675,9 +691,25 @@ CAMLprim value ocaml_vorbis_utf8_decode(value string)
   caml_raise_with_arg(*caml_named_value("vorbis_exn_utf8_failure"), string);
 }
 
-CAMLprim value ocaml_vorbis_encoder_reset(value v_e, value title, value artist, value genre, value date, value album, value tracknum) {
+CAMLprim value ocaml_vorbis_utf8_encode(value string)
+{
+  CAMLparam1(string);
+  CAMLlocal1(ret);
+
+  char* nonutf8;
+  if(utf8_encode(String_val(string), &nonutf8) >= 0)
+  {
+    ret = caml_copy_string(nonutf8);
+    free(nonutf8);
+    CAMLreturn(ret);
+  }
+  caml_raise_with_arg(*caml_named_value("vorbis_exn_utf8_failure"), string);
+}
+
+CAMLprim value ocaml_vorbis_encoder_reset(value v_e, value title, value artist, value genre, value date, value album, value tracknum, value comment)
+{
   CAMLparam5(v_e, title, artist, genre, date);
-  CAMLxparam2(album, tracknum);
+  CAMLxparam3(album, tracknum, comment);
 
   myvorbis_encoder_t *s = Encoder_val(v_e);
 
@@ -716,7 +748,7 @@ CAMLprim value ocaml_vorbis_encoder_reset(value v_e, value title, value artist,
     vorbis_comment comments;
     ogg_packet h1,h2,h3;
 
-    build_comments(&comments, string_of_option(artist), string_of_option(album), string_of_option(title), string_of_option(tracknum), string_of_option(date), string_of_option(genre));
+    build_comments(&comments, string_of_option(artist), string_of_option(album), string_of_option(title), string_of_option(tracknum), string_of_option(comment), string_of_option(date), string_of_option(genre));
 
     comments.vendor = "ocaml-vorbis";
 
@@ -747,7 +779,7 @@ CAMLprim value ocaml_vorbis_encoder_reset(value v_e, value title, value artist,
 
 value ocaml_vorbis_encoder_reset_byte(value *argv, int argc)
 {
-  return (ocaml_vorbis_encoder_reset(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]));
+  return (ocaml_vorbis_encoder_reset(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6], argv[7]));
 }
 
 CAMLprim ocaml_vorbis_get_dec_file_bitstream(value d_f)
